      logical function line_cross(aa,bb,cc,dd)
C+------------------------------------------------------------------------------
C
C     Name:   
C        line_cross
C
C     Invocation:
C        crossed = line_cross(aa,bb,cc,dd)
C
C     Description:
C        Decide whether two line segments described by their respective
C        endpoints cross.
C
C     Arguments: (">" Input, "<" Output, "W" Workspace, "!"  Changed)
C        aa:  Real array (>)
C           First endpoint of the first segment
C        bb:  Real array (>)
C           Second endpoint of the first segment
C        cc:  Real array (>)
C           First endpoint of the second segment
C        dd:  Real array (>)
C           Second endpoint of the second segment
C
C     Returns:
C        crossed:  Logical
C           TRUE if the two segments cross each other
C
C     Language:
C        FORTRAN 77
C
C     Non-standard language extensions:
C        implicit none, variable names > 6 characters
C
C     Logical i/o units used:
C        none
C
C     External routines called:
C        Library subaf2_configure:
C           cs
C           exclusive
C           rotate
C 
C     Algorithm:
C        -- Start by finding the sine and cosine of the position angle of 
C           the first segment.  
C        -- Rotate the X coordinates of the segments by this angle (about
C           the origin.
C        -- If the range of the rotated X coordinates don't overlap, then
C           the lines don't cross.
C        -- If the X ranges do overlap, then rotate the Y coordinates and
C           test their ranges.  If the rotated Y ranges don't overlap then
C           the lines don't cross.
C        -- If both of those tests fail, then repeat the exercise for the
C           position angle of the second segment.
C        -- If all four of these tests fail, then the lines cross somewhere.
C
C     Authors:
C        (JRL): James R. Lewis -- RGO, Cambridge
C
C     History:
C        01 Dec 1995 (JRL):
C           Original version
C
C-------------------------------------------------------------------------------
      implicit none
C
C     Parameters
C
      real aa(2),bb(2),cc(2),dd(2)
C
C     Local variables
C
      real costheta,sintheta,ta,tb,tc,td
      logical crossed
      integer i
C
C     Function
C
      logical exclusive
C
C     Rotate the lines by the position angle of the first line segment.
C     Start by getting the cosine and sine of the angle
C
      call cs(aa,bb,costheta,sintheta)
C
C     Loop for each x and y coordinates
C
      do 10 i = 1,2
C
C        Rotate the coordinates, i = 1 --> x, i = 2 --> y
C
         call rotate(aa,bb,cc,dd,costheta,sintheta,i,ta,tb,tc,td)
C
C        If the coordinate ranges are exclusive, then the segments don't cross
C
         if (exclusive(ta,tb,tc,td)) then
            crossed = .false.
            go to 500
         end if
  10  continue
C
C     Having got this far, try the position angle of the second segment
C
      call cs(cc,dd,costheta,sintheta)
C
C     Now loop again for x and y
C 
      do 20 i = 1,2
C
C        Rotate the  coordinates
C
         call rotate(aa,bb,cc,dd,costheta,sintheta,i,ta,tb,tc,td)
C
C        If the coordinage ranges are exclusive, then the segments  don't cross
C
         if (exclusive(ta,tb,tc,td)) then
            crossed = .false.
            go to 500
         end if
  20  continue
C
C     If we are here, then none of the exclusivity tests have passed,
C     so the lines must cross somewhere.
C   
      crossed = .true.
C
C     Exit and return the function value
C
  500 continue
      line_cross = crossed
      end

      subroutine cs(a,b,cost,sint)
C+------------------------------------------------------------------------------
C
C     Name:   
C        cs
C
C     Invocation:
C        call cs(a,b,cost,sint)
C
C     Description:
C        Calculate the sine and cosine of the position angle of a line
C        segment described by two endpoints 
C
C     Arguments: (">" Input, "<" Output, "W" Workspace, "!"  Changed)
C        a:  Real array (>)
C           First endpoint of the segment
C        b:  Real array (>)
C           Second endpoint of the segment
C        cost:  Real (<)
C           The cosine of the position angle
C        sint:  Real (<)
C           The sine of the position angle
C
C     Language:
C        FORTRAN 77
C
C     Non-standard language extensions:
C        implicit none, variable names > 6 characters
C
C     Logical i/o units used:
C        none
C
C     External routines called:
C        none
C 
C     Algorithm:
C        -- The cosine and sine are defined as: dx/r and dy/r respectively,
C           where r = sqrt(dx**2 + dy**2)
C
C     Authors:
C        (JRL): James R. Lewis -- RGO, Cambridge
C
C     History:
C        01 Dec 1995 (JRL):
C           Original version
C
C-------------------------------------------------------------------------------
      implicit none
C
C     Parameters
C
      real a(2)
      real b(2)
      real cost
      real sint
C
C     Local variables
C
      real dx,dy,rad
C
C     Calculate the trig functions of the position angle
C
      dx = b(1) - a(1)
      dy = b(2) - a(2)
      rad = sqrt(dx*dx + dy*dy)
      cost = dx/rad
      sint = dy/rad
      end

      subroutine rotate(a,b,c,d,cost,sint,mode,ta,tb,tc,td)
C+------------------------------------------------------------------------------
C
C     Name:   
C        rotate
C
C     Invocation:
C        call rotate(a,b,c,d,cost,sint,mode,ta,tb,tc,td)
C
C     Description:
C        Rotate either the X or the Y coordinates of four sets of x,y
C        coordinates with respect to the origin.
C
C     Arguments: (">" Input, "<" Output, "W" Workspace, "!"  Changed)
C        a:  Real array (>)
C           First set of unrotated x,y coordinates
C        b:  Real array (>)
C           Second set of unrotated x,y coordinates
C        c:  Real array (>)
C           Third set of unrotated x,y coordinates
C        d:  Real array (>)
C           Fourth set of unrotated x,y coordinates
C        cost:  Real (>)
C           Cosine of the angle of rotation
C        sint:  Real (>)
C           Sine of the angle of rotation
C        mode:  Integer (>)
C           Mode variable.  1 -- rotate the X coordinates, 2 -- rotate the
C           Y coordinates.
C        ta:  Real (<)
C           First rotated coordinate (either X or Y)
C        tb:  Real (<)
C           Second rotated coordinate (either X or Y)
C        tc:  Real (<)
C           Third rotated coordinate (either X or Y)
C        td:  Real (<)
C           Fourth rotated coordinate (either X or Y)
C
C     Language:
C        FORTRAN 77
C
C     Non-standard language extensions:
C        implicit none, variable names > 6 characters
C
C     Logical i/o units used:
C        none
C
C     External routines called:
C        none
C 
C     Algorithm:
C        -- If mode = 1 then calculate the rotated X coordinate by:
C           x' = x*cost + y*sint
C        -- If mode = 2 then calculate the rotated Y coordinate by:
C           y' = -x*sint + y*cost
C
C     Authors:
C        (JRL): James R. Lewis -- RGO, Cambridge
C
C     History:
C        01 Dec 1995 (JRL):
C           Original version
C
C-------------------------------------------------------------------------------
      implicit none
C
C     Parameters
C
      real a(2),b(2),c(2),d(2)
      real cost,sint
      integer mode
      real ta,tb,tc,td
C
C     If the mode is 1, then we are rotating the x coordinates
C
      if (mode .eq. 1) then
         ta = a(1)*cost + a(2)*sint
         tb = b(1)*cost + b(2)*sint
         tc = c(1)*cost + c(2)*sint
         td = d(1)*cost + d(2)*sint
C
C     Otherwise, rotate the y coordinates
C
      else 
         ta = -a(1)*sint + a(2)*cost
         tb = -b(1)*sint + b(2)*cost
         tc = -c(1)*sint + c(2)*cost
         td = -d(1)*sint + d(2)*cost
      end if
      end

      logical function exclusive(ta,tb,tc,td)
C+------------------------------------------------------------------------------
C
C     Name:   
C        exclusive
C
C     Invocation:
C        excl = exclusive(ta,tb,tc,td)
C
C     Description:
C        Test whether the ranges of two sets of coordinates overlap
C
C     Arguments: (">" Input, "<" Output, "W" Workspace, "!"  Changed)
C        ta:  Real (>)
C           First endpoint of first set
C        tb:  Real (>)
C           Second endpoint of first set
C        tc:  Real (>)
C           First endpoint of the second set
C        td:  Real (>)
C           Second endpoint of the second set
C
C     Returns:
C        excl:  Logical
C           TRUE if the ranges do _not_ overlap (i.e. they are exclusive)
C
C     Language:
C        FORTRAN 77
C
C     Non-standard language extensions:
C        implicit none, variable names > 6 characters
C
C     Logical i/o units used:
C        none
C
C     External routines called:
C        none
C 
C     Algorithm:
C        -- In order for the ranges to be exclusive, then the maximum of the
C           first set has to be less than the minimum of the second or
C           the minimum of the first set has to be greater than the maximum
C           of the second.
C
C     Authors:
C        (JRL): James R. Lewis -- RGO, Cambridge
C
C     History:
C        01 Dec 1995 (JRL):
C           Original version
C
C-------------------------------------------------------------------------------
      implicit none
C
C     Parameters
C
      real ta,tb,tc,td
C
C     See if the range of the first two arguments is exclusive of the
C     range of the second two
C
      if ((max(ta,tb) .lt. min(tc,td)) .or. 
     :   (min(ta,tb) .gt. max(tc,td))) then
         exclusive = .true.
      else
         exclusive = .false.
      end if
      end 
