      SUBROUTINE CHR_MTOE( STR1, STR2 )
*+
*  Name:
*     CHR_MTOE

*  Purpose:
*     Translate a string from the machine's character set to EBCDIC.

*  Language:
*     Starlink Fortran 77

*  Invocation:
*     CALL CHR_MTOE( STR1, STR2 )

*  Description:
*     The string STR1, which is a Fortran 77 CHARACTER string, is
*     returned in STR2 translated into a form which can be written and 
*     subsequently read correctly by a machine which uses the EBCDIC 
*     character set.

*     Any characters which are not represented in the
*     EBCDIC character set are translated to EBCDIC SPACE. Non-printable
*     characters are translated where possible.

*  Arguments:
*     STR1 = CHARACTER * ( * ) (Given)
*        The Fortran 77 character string.
*     STR2 = CHARACTER * ( * ) (Returned)
*        A character string which may be written and subsequently read 
*        correctly by a machine which uses the EBCDIC character set
*        to represent characters in Fortran. If STR2 is shorter than 
*        STR1, the translated string will be truncated; if STR2 is 
*        longer than STR1, STR2 will be padded with blanks beyond the 
*        translated string.

*  System-specific:
*     This subroutine has been implemented for machines which use
*     the ASCII character set.

*  Authors:
*     PCTR: P.C.T. Rees (STARLINK)
*     ACC:  A.C. Charles (STARLINK)
*     {enter_new_authors_here}

*  History:
*     10-JUL-1992 (PCTR):
*        Original version.
*     10-MAR-1994 (ACC):
*        Modifications to prologue.
*     {enter_changes_here}

*  Bugs:
*     {note_any_bugs_here}

*-
      
*  Type Definitions:
      IMPLICIT NONE              ! No implicit typing

*  Arguments Given:
      CHARACTER STR1 * ( * )

*  Arguments Returned:
      CHARACTER STR2 * ( * )

*  Local Constants:
      INTEGER MCHMAX             ! Maximum machine character code
      PARAMETER ( MCHMAX = 127 )

      INTEGER EBCSPC             ! EBCDIC SPACE 
      PARAMETER ( EBCSPC = 64 )

*  Local Variables:
      INTEGER ICHR               ! Character loop index
      INTEGER IMCH               ! Machine character code
      INTEGER LENGTH             ! Maximum loop index.
      INTEGER LEN1               ! Declared length of STR1
      INTEGER LEN2               ! Declared length of STR2
      INTEGER TABLE( 0 : MCHMAX ) ! Translation table

*  Local Data:
      DATA TABLE /   0,   1,   2,   3,  55,  45,  46,  47,  22,   5,
     :              37,  11,  12,  13,  14,  15,  16,  17,  18,  19,
     :              60,  61,  50,  38,  24,  25,  63,  39,  28,  29,
     :              30,  31,  64,  79, 127, 123,  91, 108,  80, 125,
     :              77,  93,  92,  78, 107,  96,  75,  97, 240, 241,
     :             242, 243, 244, 245, 246, 247, 248, 249, 122,  94,
     :              76, 126, 110, 111, 124, 193, 194, 195, 196, 197,
     :             198, 199, 200, 201, 209, 210, 211, 212, 213, 214,
     :             215, 216, 217, 226, 227, 228, 229, 230, 231, 232,
     :             233,  74, 224,  90,  95, 109, 121, 129, 130, 131,
     :             132, 133, 134, 135, 136, 137, 145, 146, 147, 148,
     :             149, 150, 151, 152, 153, 162, 163, 164, 165, 166,
     :             167, 168, 169, 192, 106, 208, 161,   7 /

*.

*  Get the declared length of the two strings.
      LEN1 = LEN( STR1 )
      LEN2 = LEN( STR2 )

*  Get the maximum loop index.
      LENGTH = MIN( LEN1, LEN2 )

*  If the string length is non-zero, loop to translate it from ASCII to
*  EBCDIC.
      IF ( LENGTH.GT.0 ) THEN

*     Initialise STR2.
         STR2 = ' '

*     Perform loop.
         DO 10 ICHR = 1, LENGTH

*        Get the machine character code.
            IMCH = ICHAR( STR1( ICHR : ICHR ) )

*        If the machine value is legitimate, translate it;
*        if not, translate it to an EBCDIC SPACE.
            IF ( IMCH.GT.MCHMAX ) THEN
               STR2( ICHR : ICHR ) = CHAR( EBCSPC )
            ELSE
               STR2( ICHR : ICHR ) = CHAR( TABLE( IMCH ) )
            END IF
 10      CONTINUE
      END IF

      END
